//===================================CISCKER====================================
//=================================Main Decoder===================================
//------------------------------------------------------------------------------
//File Name:	main_decoder.v
//Purpose:		Main decoding unit, returns the u-Instructions queue, ALU operands, and result destination
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------

module main_decoder(
			//inputs
			input clk_0,
			input final_90,
			input [7:0] opcode,
			//outputs
			output [35:0] next_queue,
			output reg [8:0] adder_alu_operands, 	//adder_A(2),adder_B(2),ALU_A(3),ALU_B(2)
			output reg [4:0] result_destination		//SP,IX,ACCA,ACCB,hidden
);

//=============u-Instructions===============
parameter p	 =	2'd0;	 //Program fetch 
parameter r	 =	2'd1;	 //Read byte 
parameter d	 =	2'd2;	 //Dummy 
parameter w	 =	2'd3;	 //Write
parameter x	 =	2'd0;	 //Don't Care

//===============Adder inputs===============
parameter adder_pc 	= 2'b00;
parameter adder_sp 	= 2'b01;
parameter adder_ix 	= 2'b10;
parameter adder_mah 	= 2'b11;

parameter zero	= 2'b00;
parameter adder_ff	= 2'b01;
parameter adder_one	= 2'b10; 
//parameter adder_mah	= 2'b11;

//================ALU inputs================
parameter ACCA 	= 3'b000;
parameter ACCB 	= 3'b001;
parameter ACCD 	= 3'b010;
parameter MM 	= 3'b011;
parameter PC	= 3'b100;
parameter SP	= 3'b101;
parameter IX	= 3'b110;
parameter ZERO_ALUA 	= 3'b111;

parameter XXX	= 3'b000;

//parameter zero	= 2'b00;
parameter accb_alub		= 2'b01;
parameter hidden 	= 2'b10;
parameter mm_alub 		= 2'b11;

//==========================Cyclic Execution Instructions Counter=======================//

reg [3:0] cycle;

wire rti_swi_trap = (opcode==8'h2D) | (opcode==8'h2F);

always@(posedge final_90)
	if(rti_swi_trap)
		cycle <= cycle+1;
	else
		cycle <= 0;
		


//======================================DECODER=========================================//

wire [3:0] opcode_h=opcode[7:4];
wire [3:0] opcode_l=opcode[3:0];

reg [11:0] u_inst=0;
reg [5:0] final=0;
reg [5:0] lookahead=0;
reg [5:0] operation=0;
reg [5:0] complement=0;

reg [8:0] operands;
reg [4:0] destination;

always@(*)

	case (opcode_h)
		4'h0:	casex (opcode_l)	//INHERENT, BRANCH
				
					4'h0:	begin 			//NOP
						u_inst 		= {x,x,x,x,x,p};
						final 	 	= 6'b000001;
						lookahead 	= 6'b000001;
						operation 	= 6'b000000;
						complement 	= 6'b000001;
						operands	= {adder_pc,zero,XXX,x};
						destination = 5'b00000;
						end
					
					//4'h1: STOP Temp
					
					4'h2: begin 			//DABNZ
						u_inst 		= {x,x,x,p,d,p};
						final 	 	= 6'b000100;
						lookahead	= 6'b000100;
						operation	= 6'b000000;
						complement	= 6'b000101; //d(pass_adder & pc_en) = branch ? 1 : 0; 
						operands	= {adder_pc,adder_mah,ACCA,zero};
						destination = 5'b00100;
						end
					
					4'h3: begin 			//DBBNZ
						u_inst 		= {x,x,x,p,d,p};
						final 	 	= 6'b000100;
						lookahead	= 6'b000100;
						operation	= 6'b000000;
						complement	= 6'b000101; //d(pass_adder & pc_en) = branch ? 1 : 0; 
						operands	= {adder_pc,adder_mah,ACCB,zero};
						destination = 5'b00010;
						end
						
					4'h4,4'h5,
					4'h6,4'h7:	begin		//BRN,BRA,BIL,BIH
						u_inst 		= {x,x,x,p,d,p};
						final 	 	= 6'b000100;
						lookahead	= 6'b000100;
						operation	= 6'b000000;
						complement	= 6'b000101;
						operands	= {adder_pc,adder_mah,XXX,x};
						destination = 5'b00000;
						end
					
					4'h8, 4'h9: 	begin  //IDIV, MUL	See Operation Decoder (CCR)
						u_inst 		= {x,x,x,x,p,d};
						complement	= 6'b000011;
						
							case (cycle)
								4'd0:	begin
										final 	 	= 6'b000001;
										lookahead	= 6'b000000;
										operation	= 6'b000000;
										operands	= {adder_pc,zero,ACCA,x};
										destination = 5'b00001;
										end
										
								4'd1:	begin
										final 	 	= 6'b000001;
										lookahead	= 6'b000000;
										operation	= 6'b000001;
										operands	= {adder_pc,zero,ZERO_ALUA,x};
										destination = 5'b00100;
										end
								4'd7:	begin
										final 	 	= 6'b000010;
										lookahead	= 6'b000010;
										operation	= 6'b000011;
										operands	= {adder_pc,zero,ACCA,x};
										destination = 5'b00100;
										end
								default:begin
										final 	 	= 6'b000001;
										lookahead	= 6'b000000;
										operation	= 6'b000001;
										operands	= {adder_pc,zero,ACCA,x};
										destination = 5'b00100;
										end
							endcase
						end
					
						
					default:	begin	//CLV,SEV,CLC,SEC,CLI,SEI
						u_inst 		= {x,x,x,x,x,p};
						final 	 	= 6'b000001;
						lookahead 	= 6'b000001;
						operation 	= 6'b000000;
						complement 	= 6'b000001;
						operands	= {adder_pc,zero,XXX,x};
						destination = 5'b00000;
						end
				endcase
						
		4'h1:	begin 				//BRANCH
				u_inst 		= {x,x,x,p,d,p}; //BHCC,BHCS,BHI,BLS,BCC,BCS,BNE,BEQ,BVC,BVS,BPL,BMI,BGE,BLT,BGT,BLE
				final 	 	= 6'b000100;
				lookahead	= 6'b000100;
				operation	= 6'b000000;
				complement	= 6'b000101;		// d(pass_adder & pc_en) = branch ? 1 : 0; 
				operands	= {adder_pc,adder_mah,XXX,x};
				destination = 5'b00000;
				end
					
		4'h2:	casex (opcode_l)	//INHERENT
					
					4'h6:	begin 		//PULX
						u_inst 		= {x,x,x,p,d,d};
						final 	 	= 6'b000100;
						lookahead 	= 6'b000001;
						operation 	= 6'b000100;
						complement 	= 6'b000011;
						operands	= {adder_sp,adder_one,MM,x};
						destination = 5'b11000;
						end
				
					4'h7:	begin 		//PSHX
						u_inst 		= {x,x,x,w,w,d};
						final 	 	= 6'b000100;
						lookahead 	= 6'b000001;
						operation 	= 6'b000011;
						complement 	= 6'b000100;
						operands	= {adder_sp,adder_ff,IX,x};
						destination = 5'b10000;
						end
					
					4'h8,4'h9:	begin 	//PULA,PULB
						u_inst 		= {x,x,x,x,p,d};
						final 	 	= 6'b000010;
						lookahead 	= 6'b000001;
						operation 	= 6'b000010;
						complement 	= 6'b000011;
						operands	= {adder_sp,adder_one,XXX,x};
							if  (opcode_l[0])	//PULB
								destination = 5'b10010;				
							else					//PULA
								destination = 5'b10100;
						end
						
					4'hA,4'hB:	begin 	//PSHA,PSHB
						u_inst 		= {x,x,x,x,w,d};
						final 	 	= 6'b000010;
						lookahead 	= 6'b000001;
						operation 	= 6'b000000;
						complement 	= 6'b000010;
						destination = 5'b10000;
							if (opcode_l[0])	//PSHB
								operands	= {adder_sp,adder_ff,ACCB,x};
							else					//PSHA
								operands	= {adder_sp,adder_ff,ACCA,x};
						end
					
					4'hC: begin 		//RTS
						u_inst 		= {x,r,d,p,r,r};
						final 	 	= 6'b010000;
						lookahead 	= 6'b010000;
						operation 	= 6'b000011;
						complement 	= 6'b011000;
						operands	= {x,x,SP,zero};
						destination = 5'b10000;
						end
						
					4'hD: 				//RTI
						case(cycle)
							2'd0:begin					//RTI = PULL CCR,ACCA
								u_inst 		= {x,x,x,p,r,r};
								final 	 	= 6'b000100;
								lookahead 	= 6'b000000;
								operation 	= 6'b000100;
								complement 	= 6'b000100;
								operands	= {adder_sp,adder_ff,MM,x};
								destination = 5'b00100;
								end
							2'd1:begin					//RTI = PULL IX
								u_inst 		= {x,x,x,p,r,r};
								final 	 	= 6'b000100;
								lookahead 	= 6'b000000;
								operation 	= 6'b000100;
								complement 	= 6'b000100;
								operands	= {adder_sp,adder_ff,MM,x};
								destination = 5'b01000;
								end
							2'd2:begin					//RTI = PULL PC
								u_inst 		= {x,x,x,p,r,r};
								final 	 	= 6'b000100;
								lookahead 	= 6'b000000;
								operation 	= 6'b000100;
								complement 	= 6'b000100;
								operands	= {adder_sp,adder_ff,MM,x};
								destination = 5'b00000;
								end
							2'd3:begin					//RTI = JUMP
								u_inst 		= {x,x,x,x,r,d};
								final 	 	= 6'b000010;
								lookahead 	= 6'b000001;
								operation 	= 6'b000011;
								complement 	= 6'b000011;
								operands	= {adder_mah,zero,x,x};
								destination = 5'b00000;
								end
						endcase

					//4'hE: WAIT	Temp
					4'hF: 				//SWI
						case(cycle)
							2'd0:begin					//SWI = PUSH PC
								u_inst 		= {x,x,x,w,w,d};
								final 	 	= 6'b000100;
								lookahead 	= 6'b000000;
								operation 	= 6'b000011;
								complement 	= 6'b000100;
								operands	= {adder_sp,adder_ff,PC,x};
								destination = 5'b00000;
								end
							2'd1:begin					//SWI = PUSH IX
								u_inst 		= {x,x,x,w,w,d};
								final 	 	= 6'b000100;
								lookahead 	= 6'b000000;
								operation 	= 6'b000011;
								complement 	= 6'b000100;
								operands	= {adder_sp,adder_ff,IX,x};
								destination = 5'b00000;
								end
							2'd2:begin					//SWI = PUSH CCR,ACCA
								u_inst 		= {x,x,x,w,w,d};
								final 	 	= 6'b000100;
								lookahead 	= 6'b000000;
								operation 	= 6'b000011;
								complement 	= 6'b000100;
								operands	= {adder_sp,adder_ff,ACCD,x};
								destination = 5'b00000;
								end
							2'd3:begin					//SWI = JUMP
								u_inst 		= {x,x,r,d,p,p};
								final 	 	= 6'b001000;
								lookahead 	= 6'b001000;
								operation 	= 6'b000000;
								complement 	= 6'b001100;
								operands	= {adder_mah,zero,XXX,x};
								destination = 5'b00000;
								end
						endcase
					
					default:	begin 	//TSX,TXS,INS,DES,INX,DEX
						u_inst 		= {x,x,x,x,x,p};
						final 	 	= 6'b000001;
						lookahead 	= 6'b000001;
						operation 	= 6'b000001;
						complement 	= 6'b000001;
						
							case (opcode_l[2:0])
								3'h0:begin
										operands	= {adder_pc,zero,SP,zero}; //TSX
										destination = 5'b01000; 
									end
								3'h1:begin
										operands	= {adder_pc,zero,IX,zero}; //TXS
										destination = 5'b10000; 
									end
								3'h2,3'h3:begin 						//INS,DES
										operands	= {adder_pc,zero,SP,zero};
										destination = 5'b10000;
									end
								3'h4,3'h5:begin 						//INX,DEX
										operands	= {adder_pc,zero,IX,zero};
										destination = 5'b01000;
									end
																
								default: begin //Temp
											operands	= {adder_pc,zero,SP,zero};
											destination = 5'b10000;
										end
							endcase
						end
				endcase
								
		4'h3:	begin				//INHERENT ACCA
					u_inst 		= {x,x,x,x,x,p}; //NEGA,CLRA,INCA,DECA,LSRA,RORA,ASRA,LSLA,ROLA,TSTA,TDX,TBA,SBA,ABA
		            final 	 	= 6'b000001;
		            lookahead 	= 6'b000001; 	
		            operation 	= 6'b000001; 	
		            complement 	= 6'b000001;

						casex (opcode_l)
												
							4'h9:	begin			//TSTA
								operands	= {adder_pc,zero,ACCA,x};
								destination = 5'b00000;
								end
					
							4'hA:	begin			//TDX
								operands	= {adder_pc,zero,ACCD,x};
								destination = 5'b01000;
								end
						
							4'hC:	begin			//TBA
								operands	= {adder_pc,zero,ACCB,x};
								destination = 5'b00100;
								end
					
							4'hE,4'hF:	begin		//SBA,ABA
								operands	= {adder_pc,zero,ACCA,accb_alub};
								destination = 5'b00100;
								end

							default:	begin 		//NEGA,CLRA,INCA,DECA,LSRA,RORA,ASRA,LSLA,ROLA
								operands	= {adder_pc,zero,ACCA,zero};
								destination = 5'b00100;
								end
						endcase
				end	
					
		4'h4:	begin				//INHERENT ACCB
					u_inst 		= {x,x,x,x,x,p}; //NEGB,CLRB,INCB,DECB,LSRB,RORB,ASRB,LSLB,ROLB,TSTB,TAB,CBA
					final 	 	= 6'b000001;
					lookahead 	= 6'b000001;
					operation 	= 6'b000001;
					complement 	= 6'b000001;
						casex (opcode_l)
							
							4'h9:	begin			//TSTB
								operands	= {adder_pc,zero,ACCB,x};
								destination = 5'b00000;
								end				
							4'hC:	begin			//TAB
								operands	= {adder_pc,zero,ACCA,x};
								destination = 5'b00010;
								end
							4'hE:	begin			//CBA
								operands	= {adder_pc,zero,ACCA,accb_alub};
								destination = 5'b00000;
								end
							default: begin	//NEGB,CLRB,INCB,DECB,LSRB,RORB,ASRB,LSLB,ROLB
								operands	= {adder_pc,zero,ACCB,zero};
								destination = 5'b00010;
								end
						endcase
				end
				
		4'h5:	casex (opcode_l)	//INDEXED w/o ACCX
					
					4'h9:	begin		//TST
						u_inst 		= {x,x,p,d,p,p};
						final 	 	= 6'b001000;
						lookahead 	= 6'b000100;
						operation 	= 6'b001000;
						complement 	= 6'b001100;
						operands	= {adder_mah,zero,MM,x};
						destination = 5'b00000;
						end
					
					4'hA,4'hB:	begin		//BRSET,BRCLR
						u_inst 		= {p,d,p,p,d,p};
						final 	 	= 6'b100000;
						lookahead 	= 6'b100000;
						operation 	= 6'b000100;
						complement 	= 6'b101111;
						destination = 5'b00001;
						operands = {adder_ix,adder_mah,MM,hidden};
						end
					
					4'hC,4'hD:	begin	//BCLR,BSET
						u_inst 		= {x,p,w,d,d,p};
						final 	 	= 6'b010000;
						lookahead 	= 6'b010000;
						operation 	= 6'b000100;
						complement 	= 6'b011111;
						operands	= {adder_ix,adder_mah,MM,hidden};
						destination = 5'b00001;
						end
					
					4'hE: begin			//MOV
						u_inst 		= {w,d,p,d,d,p};
						final 	 	= 6'b100000;
						lookahead 	= 6'b010000;
						operation 	= 6'b011000;
						complement 	= 6'b111011;
						operands	= {adder_ix,adder_mah,MM,hidden};
						destination = 5'b00001;
						end
					
					4'hF:	begin		//JMP
						u_inst 		= {x,x,x,r,d,p};	
						final 	 	= 6'b000100;
						lookahead 	= 6'b000100;
						operation 	= 6'b000000;
						complement 	= 6'b000111;
						operands	= {adder_ix,adder_mah,XXX,x};
						destination = 5'b00000;
						end
					
					default: begin		//NEG,CLR,INC,DEC,LSR,ROR,ASR,LSL,ROL
						u_inst 		= {x,x,w,d,d,p};
						final 	 	= 6'b001000;
						lookahead 	= 6'b000010;
						operation 	= 6'b001000;
						complement 	= 6'b001111;
						operands	= {adder_ix,adder_mah,MM,zero};
						destination = 5'b00000;
						end
				endcase
			
		4'h6:	casex (opcode_l)	//EXTENDED w/o ACCX
					
					4'h9:	begin		//TST
						u_inst 		= {x,x,p,d,p,p};
						final 	 	= 6'b001000;		
						lookahead 	= 6'b000100;
						operation 	= 6'b001000;
						complement 	= 6'b001100;
						operands	= {adder_mah,zero,MM,x};
						destination = 5'b00000;
						end
					
					4'hA,4'hB:	begin		//BRSET,BRCLR DIRECT
						u_inst 		= {p,d,p,p,d,p};
						final 	 	= 6'b100000;
						lookahead 	= 6'b100000;
						operation 	= 6'b000100;
						complement 	= 6'b101111;
						destination = 5'b00001;
						operands 	= {adder_mah,zero,MM,hidden};
						end
										
					4'hC,4'hD:	begin	//BCLR,BSET
						u_inst 		= {p,w,d,d,p,p};
						final 	 	= 6'b100000;		
						lookahead 	= 6'b100000;
						operation 	= 6'b001000;
						complement 	= 6'b111100;
						operands	= {adder_mah,zero,MM,hidden};
						destination = 5'b00001;
						end
					
					4'hE: begin	//MOV
						u_inst 		= {w,d,p,d,d,p};
						final 	 	= 6'b100000;
						lookahead 	= 6'b010000;
						operation 	= 6'b011000;
						complement 	= 6'b111011;
						operands	= {adder_mah,zero,ZERO_ALUA,hidden};
						destination = 5'b00001;
						end
					
					4'hF:	begin		//JMP
						u_inst 		= {x,x,r,d,p,p};	
						final 	 	= 6'b001000;
						lookahead 	= 6'b001000;
						operation 	= 6'b000000;
						complement 	= 6'b001100;
						operands	= {adder_mah,zero,XXX,x};
						destination = 5'b00000;
						end
					default: begin		//NEG,CLR,INC,DEC,LSR,ROR,ASR,LSL,ROL
						u_inst 		= {x,w,d,d,p,p};	
						final 	 	= 6'b010000;
						lookahead 	= 6'b000100;
						operation 	= 6'b010000;
						complement 	= 6'b011100;
						operands	= {adder_mah,zero,MM,zero};
						destination = 5'b00000;
						end
				endcase
			
		4'h7:	casex (opcode_l)	//INDEXED w/ Post-increment
					
					4'hA:	begin	//STA
						u_inst 		= {x,x,x,x,w,d};
						final 	 	= 6'b000010;
						lookahead 	= 6'b000001;
						operation 	= 6'b000010;
						complement 	= 6'b000010;
						operands = {adder_ix,adder_one,ACCA,x};
						destination = 5'b01000;
						end
						
					4'hD,4'hE,4'hF:	begin //SUBD,CPD,ADDD
						u_inst 		= {x,x,x,p,d,d};
						final 	 	= 6'b000100;
						lookahead 	= 6'b000001;
						operation 	= 6'b000100;
						complement 	= 6'b000000;
						operands = {adder_ix,adder_one,ACCD,x};
							if (opcode[1:0]==2'b10)
								destination = 5'b01000;
							else
								destination = 5'b01110;
						end
					
					default: begin ////SUBA,SBCA,ADDA,ANDA,LDAA,ORAA,EORA,ADCA,CMPA,BITA
						u_inst 		= {x,x,x,x,p,d};
						final 	 	= 6'b000010;
						lookahead 	= 6'b000001;
						operation 	= 6'b000010;
						complement 	= 6'b000010;
							if (opcode_l[2:0]==3'b100)	//LDAA
								operands = {adder_ix,adder_one,MM,x};
							else						//SUBA,SBCA,ADDA,ANDA,ORAA,EORA,ADCA,CMPA,BITA
								operands = {adder_ix,adder_one,ACCA,mm_alub};
							
							if (opcode_l[3]==1'b0)		//SUBA,SBCA,ADDA,ANDA,LDAA,ORAA,EORA,ADCA
								destination = 5'b01100;	
							else						//CMPA,BITA
								destination = 5'b01000;
						end
				endcase
			
		4'h8: 	casex (opcode_l)	//IMMEDIATE ACCA
					
					4'hB: begin	//AIS
						u_inst 		= {x,x,x,p,p,p};	
						final 	 	= 6'b000100;
						lookahead 	= 6'b000100;
						operation 	= 6'b000010;
						complement 	= 6'b000000;
						operands = {adder_pc,zero,SP,mm_alub};
						destination = 5'b10000;
						end
					
					4'hD,4'hE,4'hF: begin
						u_inst 		= {x,x,x,p,p,p};	
						final 	 	= 6'b000100;
						lookahead 	= 6'b000100;
						operation 	= 6'b000010;
						complement 	= 6'b000000;
							if(opcode[1])							//SUBD,ADDD
								begin
									operands = {adder_pc,zero,ACCD,mm_alub};		
									destination = 5'b00110;
								end
							else
								begin
									operands = {adder_pc,zero,MM,zero};	//LDS
									destination = 5'b10000;
								end
						end		
								
					default: begin //SUBA,SBCA,ADDA,ANDA,LDAA,EORA,ADCA,ORAA,CMPA,BITA
						u_inst 		= {x,x,x,x,p,p};
						final 	 	= 6'b000010;
						lookahead 	= 6'b000010;
						operation 	= 6'b000001;
						complement 	= 6'b000011;
							if (opcode_l[2:0]==3'b100)	//LDAA
								operands	= {adder_pc,zero,MM,mm_alub};
							else						//SUBA,SBCA,ADDA,ANDA,EORA,ADCA,ORAA,CMPA,BITA
								operands	= {adder_pc,zero,ACCA,mm_alub};
								
							if (opcode_l[3:1]==3'b100)	//CMPA,BITA
								destination = 5'b00000;
							else						//SUBA,SBCA,ADDA,ANDA,LDAA,EORA,ADCA,ORAA
								destination = 5'b00100;	
						end
					endcase
					
		4'h9:	casex (opcode_l)	//DIRECT ACCA
					
					4'hA: begin	//STA
						u_inst 		= {x,x,x,w,d,p};	
						final 	 	= 6'b000100;		
						lookahead 	= 6'b000010;
						operation 	= 6'b000100;
						complement 	= 6'b000111;
						destination = 5'b00000;
						operands	= {adder_mah,zero,ACCA,zero};
						end

					4'hB,4'hC: begin	//STS,STD
						u_inst 		= {x,x,w,w,d,p};	
						final 	 	= 6'b001000;		
						lookahead 	= 6'b000010;
						operation 	= 6'b001000;
						complement 	= 6'b001011;
						destination = 5'b00000;
							if(opcode_l[0])	//STS
								operands	= {adder_mah,zero,SP,zero};
							else			//STD
								operands	= {adder_mah,zero,ACCD,zero};
						end
					
					4'hD,4'hE,4'hF: begin
						u_inst 		= {x,x,p,r,d,p};	
						final 	 	= 6'b001000;		
						lookahead 	= 6'b000010;
						operation 	= 6'b001000;
						complement 	= 6'b000000;
						operands	= {adder_mah,zero,ACCD,mm_alub};
							if(opcode_l[3:1]==3'b111)	//SUBD,ADDD
								destination = 5'b00110;
							else						//LDS
								destination = 5'b10000;
						end
						
					default: begin		//SUBA,SBCA,ADDA,ANDA,LDAA,EORA,ADCA,ORAA,CMPA,BITA
						u_inst 		= {x,x,x,p,d,p};	
						final 	 	= 6'b000100;		
						lookahead 	= 6'b000010;
						operation 	= 6'b000100;
						complement 	= 6'b000000;
							if (opcode_l[2:0]==3'b100)	//LDAA
								operands	= {adder_mah,zero,MM,mm_alub};
							else						//SUBA,SBCA,ADDA,ANDA,EORA,ADCA,ORAA,CMPA,BITA
								operands	= {adder_mah,zero,ACCA,mm_alub};
								
							if (opcode_l[3]==1'b0)		//SUBA,SBCA,ADDA,ANDA,LDAA,EORA,ADCA,ORAA
								destination = 5'b00100;
							else						//CMPA,BITA
								destination = 5'b00000;
						end
				endcase
								
		4'hA:	casex (opcode_l)	//INDEXED ACCA
					
					4'hA: begin	//STA
						u_inst 		= {x,x,x,w,d,p};	
						final 	 	= 6'b000100;		
						lookahead 	= 6'b000010;
						operation 	= 6'b000100;
						complement 	= 6'b000111;
						destination = 5'b00000;
						operands	= {adder_ix,adder_mah,ACCA,zero};
						end
					
					4'hB,4'hC: begin	//STS,STD
						u_inst 		= {x,x,w,w,d,p};	
						final 	 	= 6'b001000;		
						lookahead 	= 6'b000010;
						operation 	= 6'b001000;
						complement 	= 6'b001011;
						destination = 5'b00000;
							if(opcode_l[0])	//STS
								operands	= {adder_ix,adder_mah,SP,zero};
							else			//STD
								operands	= {adder_ix,adder_mah,ACCD,zero};
						end
					
					4'hD,4'hE,4'hF: begin //LDS,SUBD,ADDD
						u_inst 		= {x,x,p,r,d,p};
						final 	 	= 6'b001000;		
						lookahead 	= 6'b000010;
						operation 	= 6'b001000;
						complement 	= 6'b000011;
							if(opcode_l[1]==1'b0)	//LDS
								begin
									operands	= {adder_ix,adder_mah,MM,zero};
									destination = 5'b10000;
								end
							else					//SUBD,ADDD
								begin
									operands	= {adder_ix,adder_mah,ACCD,mm_alub};
									destination = 5'b00110;
								end
						end
						
					default: begin
						u_inst 		= {x,x,x,p,d,p};	
						final 	 	= 6'b000100;		
						lookahead 	= 6'b000010;
						operation 	= 6'b000100;
						complement 	= 6'b000000;
							if (opcode_l[2:0]==3'b100)		//LDAA
								operands	= {adder_ix,adder_mah,MM,mm_alub};	
							else							//SUBA,SBCA,ADDA,ANDA,EORA,ADCA,ORAA,CMPA,BITA
								operands	= {adder_ix,adder_mah,ACCA,mm_alub};
							
							if (opcode_l[3]==1'b0)		//SUBA,SBCA,ADDA,ANDA,LDAA,EORA,ADCA,ORAA
								destination = 5'b00100;
							else						//CMPA,BITA
								destination = 5'b00000;

						end

				endcase
				
		4'hB:	casex (opcode_l)	//EXTENDED ACCA
					
					4'hA: begin	//STA
						u_inst 		= {x,x,w,d,p,p};	
						final 	 	= 6'b001000;		
						lookahead 	= 6'b000100;
						operation 	= 6'b001000;
						complement 	= 6'b001100;
						destination = 5'b00000;
						operands	= {adder_mah,zero,ACCA,zero};
						end
						
					4'hB,4'hC: begin	//STS,STD
						u_inst 		= {x,w,w,d,p,p};	
						final 	 	= 6'b010000;		
						lookahead 	= 6'b000100;
						operation 	= 6'b010000;
						complement 	= 6'b010100;
						destination = 5'b00000;
							if(opcode_l[0])	//STS
								operands	= {adder_mah,zero,SP,zero};
							else			//STD
								operands	= {adder_mah,zero,ACCD,zero};
						end
						
					4'hD,4'hE,4'hF: begin	//LDS,SUBD,ADDD
						u_inst 		= {x,p,r,d,p,p};	
						final 	 	= 6'b010000;		
						lookahead 	= 6'b000100;
						operation 	= 6'b010000;
						complement 	= 6'b000100;
							if(opcode_l[1]==1'b0)	//LDS
								begin
									operands	= {adder_mah,zero,MM,zero};
									destination = 5'b10000;
								end
							else					//SUBD,ADDD
								begin
									operands	= {adder_mah,zero,ACCD,mm_alub};
									destination = 5'b00110;
								end
						end
					
					default: begin	
						u_inst 		= {x,x,p,d,p,p};
						final 	 	= 6'b001000;		
						lookahead 	= 6'b000100;
						operation 	= 6'b001000;
						complement 	= 6'b001000;
							if(opcode_l[2:0]==3'b100)	//LDAA
								operands	= {adder_mah,zero,MM,mm_alub};
							else						//SUBA,SBCA,ADDA,ANDA,EORA,ADCA,ORAA,CMPA,BITA
								operands	= {adder_mah,zero,ACCA,mm_alub};
								
							if(opcode_l[3])				//CMPA,BITA
								destination = 5'b00000;
							else						//SUBA,SBCA,ADDA,ANDA,LDAA,EORA,ADCA,ORAA
								destination = 5'b00100;
						end	
					
					endcase
			
		4'hC: 	casex (opcode_l)	//IMMEDIATE ACCB
					
					4'h9: begin	//BSR
						u_inst 		= {x,p,w,w,d,p};	
						final 	 	= 6'b010000;		
						lookahead 	= 6'b010000;
						operation 	= 6'b000110;
						complement 	= 6'b011001;
						destination = 5'b10000;
						operands	= {adder_sp,adder_ff,PC,zero};
						end
					
					4'hB: begin	//AIX
						u_inst 		= {x,x,x,p,p,p};
						final 	 	= 6'b000100;
						lookahead 	= 6'b000100;
						operation 	= 6'b000010;
						complement 	= 6'b000000;
						operands	= {adder_pc,zero,IX,mm_alub};
						destination = 5'b01000;
						end
					
					4'hC,4'hD,4'hE,4'hF: begin	//CPX,LDX,CPD,LDD
						u_inst 		= {x,x,x,p,p,p};
						final 	 	= 6'b000100;
						lookahead 	= 6'b000100;
						operation 	= 6'b000010;
						complement 	= 6'b000000;
							case (opcode_l[1:0])
								2'b00:	begin						//CPX
											operands	= {adder_pc,zero,IX,mm_alub};
											destination = 5'b00000;
										end
								
								2'b01:	begin						//LDX
											operands	= {adder_pc,zero,MM,mm_alub};
											destination = 5'b01000;
										end
								
								2'b10:	begin						//CPD
											operands	= {adder_pc,zero,ACCD,mm_alub};
											destination = 5'b00000;
										end
								
								2'b11:	begin						//LDD
											operands	= {adder_pc,zero,MM,mm_alub};
											destination = 5'b00110;
										end
								default: begin						//Temp
											operands	= {adder_pc,zero,ACCD,mm_alub};
											destination = 5'b00000;
										end
							endcase
						end
					
					default: begin 		//SUBB,SBCB,ADDB,ANDB,LDAB,ORAB,EORB,ADCB,CMPB
						u_inst	 	= {x,x,x,p,p};	
						final 	 	= 6'b000010;
						lookahead 	= 6'b000010;
						operation 	= 6'b000001;
						complement 	= 6'b000011;
							if (opcode_l == 4'h4)				//LDAB
								operands	= {adder_pc,zero,MM,mm_alub};	
							else								//SUBB,SBCB,ADDB,ANDB,ORAB,EORB,ADCB,CMPB
								operands	= {adder_pc,zero,ACCB,mm_alub};
								
							if (opcode_l == 4'h8)				//CMPB
								destination = 5'b00000;			
							else								//SUBB,SBCB,ADDB,ANDB,LDAB,ORAB,EORB,ADCB
								destination = 5'b00010;
						end
				endcase
						
		4'hD:	casex (opcode_l)	//DIRECT ACCB
					
					4'h9: begin	//JSR
						u_inst 		= {x,p,w,w,d,p};	
						final 	 	= 6'b010000;		
						lookahead 	= 6'b010000;
						operation 	= 6'b000110;
						complement 	= 6'b011001;
						destination = 5'b10000;
						operands	= {adder_sp,adder_ff,PC,zero};
						end
					
					4'hA: begin	//STB
						u_inst 		= {x,x,x,w,d,p};	
						final 	 	= 6'b000100;		
						lookahead 	= 6'b000010;
						operation 	= 6'b000100;
						complement 	= 6'b000111;
						destination = 5'b00000;
						operands	= {adder_mah,zero,ACCB,zero};
						end
					
					4'hB: begin	//STX
						u_inst 		= {x,x,w,w,d,p};	
						final 	 	= 6'b001000;		
						lookahead 	= 6'b000010;
						operation 	= 6'b001000;
						complement 	= 6'b001011;
						destination = 5'b00000;
						operands	= {adder_mah,zero,IX,zero};
						end
					
					4'hC,4'hD,4'hE,4'hF: begin	//CPX, LDX, CPD, LDD
						u_inst 		= {x,x,p,r,d,p};
					    final 	 	= 6'b001000;
					    lookahead 	= 6'b000010;
					    operation 	= 6'b001000;
					    complement 	= 6'b000111;
							case (opcode_l[1:0])
								2'b00:	begin	//CPX
											operands	= {adder_mah,zero,IX,mm_alub};
											destination = 5'b00000;
										end
								2'b01:	begin	//LDX
											operands	= {adder_mah,zero,MM,mm_alub};
											destination = 5'b01000;
										end
								2'b10:	begin	//CPD
											operands	= {adder_mah,zero,ACCD,mm_alub};
											destination = 5'b00000;
										end
								2'b11:	begin	//LDD
											operands	= {adder_mah,zero,MM,mm_alub};
											destination = 5'b00110;
										end
								default: begin	//Temp
											operands	= {adder_mah,zero,MM,mm_alub};
											destination = 5'b00110;
										end
							endcase
						end
						
					default: begin
						u_inst 		= {x,x,x,p,d,p};
						final 	 	= 6'b000100;
						lookahead 	= 6'b000010;
						operation 	= 6'b000100;
						complement 	= 6'b000111;
							if (opcode_l == 4'h4)	//LDAB
								operands	= {adder_mah,zero,MM,mm_alub};
							else							//SUBB,SBCB,ADDB,ANDB,ORAB,EORB,ADCB,CMPB
								operands	= {adder_mah,zero,ACCB,mm_alub};
							
							if (opcode_l == 4'h8)	//CMPB
								destination = 5'b00000;
							else							//SUBB,SBCB,ADDB,ANDB,LDAB,ORAB,EORB,ADCB
								destination = 5'b00010;
						end		
					
				endcase
									
		4'hE: 	casex (opcode_l)	//INDEXED ACCB
					
					4'h9: begin	//JSR
						u_inst 		= {x,p,w,w,d,p};	
						final 	 	= 6'b010000;		
						lookahead 	= 6'b010000;
						operation 	= 6'b000110;
						complement 	= 6'b011001;
						destination = 5'b10000;
						operands	= {adder_sp,adder_ff,PC,zero};
						end
					
					4'hA: begin	//STB
						u_inst 		= {x,x,x,w,d,p};	
						final 	 	= 6'b000100;		
						lookahead 	= 6'b000010;
						operation 	= 6'b000100;
						complement 	= 6'b000111;
						destination = 5'b00000;
						operands	= {adder_ix,adder_mah,ACCB,zero};
						end
					
					4'hB: begin	//STX
						u_inst 		= {x,x,w,w,d,p};
						final 	 	= 6'b001000;	
						lookahead 	= 6'b000010;
						operation 	= 6'b001000;
						complement 	= 6'b001011;
						operands	= {adder_ix,adder_mah,IX,x};
						destination = 5'b00000;
						end
											
					4'hC,4'hD,4'hE,4'hF: begin //CPX,LDX,CPD,LDD
						u_inst 		= {x,x,p,r,d,p};	
						final 	 	= 6'b001000;
						lookahead 	= 6'b000010;
						operation 	= 6'b010000;
						complement 	= 6'b000011;
							case (opcode_l[1:0])
								2'b00:	begin	//CPX
											operands	= {adder_ix,adder_mah,IX,mm_alub};
											destination = 5'b00000;
										end
								2'b01:	begin	//LDX
											operands	= {adder_ix,adder_mah,MM,mm_alub};
											destination = 5'b01000;
										end
								2'b10:	begin	//CPD
											operands	= {adder_ix,adder_mah,ACCD,mm_alub};
											destination = 5'b00000;
										end
								2'b11:	begin	//LDD
											operands	= {adder_ix,adder_mah,MM,mm_alub};
											destination = 5'b00110;
										end
							endcase						
						end
					
					default: begin	//SUBB,SBCB,ADDB,ANDB,LDAB,ORAB,EORB,ADCB,CMPB
						u_inst 		= {x,x,x,p,d,p};
						final 	 	= 6'b000100;
						lookahead 	= 6'b000010;
						operation 	= 6'b000100;
						complement 	= 6'b000000;
							if (opcode_l == 4'h4)	//LDAB
								operands	= {adder_ix,adder_mah,MM,mm_alub};
							else							//SUBB,SBCB,ADDB,ANDB,ORAB,EORB,ADCB,CMPB
								operands	= {adder_ix,adder_mah,ACCB,mm_alub};
							
							if (opcode_l == 4'h8)	//CMPB
								destination = 5'b00000;
							else							//SUBB,SBCB,ADDB,ANDB,LDAB,ORAB,EORB,ADCB
								destination = 5'b00110;
						end		
					
				endcase
						
		4'hF:	casex (opcode_l)	//EXTENDED ACCB
					
					4'h9: begin //JSR
						u_inst 		= {p,w,w,d,p,p};	
						final 	 	= 6'b100000;		
						lookahead 	= 6'b100000;
						operation 	= 6'b001100;
						complement 	= 6'b110000;
						destination = 5'b10000;
						operands	= {adder_sp,adder_ff,PC,zero};
						end
					
					4'hA: begin	//STB
						u_inst 		= {x,x,w,d,p,p};	
						final 	 	= 6'b001000;		
						lookahead 	= 6'b000100;
						operation 	= 6'b001000;
						complement 	= 6'b001100;
						destination = 5'b00000;
						operands	= {adder_mah,zero,ACCB,zero};
						end	
						
					4'hB: begin	//STX
						u_inst 		= {x,w,w,d,p,p};	
						final 	 	= 6'b010000;		
						lookahead 	= 6'b000100;
						operation 	= 6'b010000;
						complement 	= 6'b010100;
						destination = 5'b00000;
						operands	= {adder_mah,zero,IX,zero};
						end
					
					4'hC,4'hD,4'hE,4'hF: begin	//CPX,LDX,CPD,LDD
						u_inst 		= {x,p,r,d,p,p};	
						final 	 	= 6'b010000;
						lookahead 	= 6'b000100;
						operation 	= 6'b010000;
						complement 	= 6'b000100;
							case (opcode_l[1:0])
								2'b00:	begin	//CPX
											operands	= {adder_mah,zero,IX,mm_alub};
											destination = 5'b00000;
										end
								2'b01:	begin	//LDX
											operands	= {adder_mah,zero,MM,mm_alub};
											destination = 5'b01000;
										end
								2'b10:	begin	//CPD
											operands	= {adder_mah,zero,ACCD,mm_alub};
											destination = 5'b00000;
										end
								2'b11:	begin	//LDD
											operands	= {adder_mah,zero,MM,mm_alub};
											destination = 5'b00110;
										end
							endcase
						end
						
					default:	begin	//SUBB,SBCB,ADDB,ANDB,LDAB,ORAB,EORB,ADCB,CMPB
						u_inst 		= {x,x,p,d,p,p};			
						final 	 	= 6'b001000;
						lookahead 	= 6'b000100;
						operation 	= 6'b001000;
						complement 	= 6'b001000;
							if (opcode_l == 4'h4)	//LDAB
								operands	= {adder_mah,zero,MM,mm_alub};
							else						//SUBB,SBCB,ADDB,ANDB,ORAB,EORB,ADCB,CMPB
								operands	= {adder_mah,zero,ACCB,mm_alub};
							
							if (opcode_l == 4'h8)		//CMPB
								destination = 5'b00000;
							else						//SUBB,SBCB,ADDB,ANDB,LDAB,ORAB,EORB,ADCB
								destination = 5'b00010;
						end		
				endcase
		
		endcase

//===================================OUTPUT ASSIGNATION=====================================//

assign next_queue[5:0]		={final[0],lookahead[0],operation[0],complement[0],u_inst[1:0]};
assign next_queue[11:6]		={final[1],lookahead[1],operation[1],complement[1],u_inst[3:2]};
assign next_queue[17:12]	={final[2],lookahead[2],operation[2],complement[2],u_inst[5:4]};
assign next_queue[23:18]	={final[3],lookahead[3],operation[3],complement[3],u_inst[7:6]};
assign next_queue[29:24]	={final[4],lookahead[4],operation[4],complement[4],u_inst[9:8]};
assign next_queue[35:30]	={final[5],lookahead[5],operation[5],complement[5],u_inst[11:10]};

always@(posedge clk_0)
if(final_90)
begin
	adder_alu_operands	<=	operands;
	result_destination 	<=	destination;
end

endmodule
